#!/bin/sh

iraf_pkg()
{
  local PKG_NAME EXTERNPKG_FILE MODE MKPKG_NAME

  #
  #  Various settings
  #

  EXTERNPKG_FILE=$SCISOFT_SHARE/iraf/iraf/unix/hlib/extern.pkg

  #
  #  Process options
  #

  MODE=source
  MKPKG_NAME=unset
  while [ "X$1" != X ]; do
  case $1 in
    --edit)       MODE=edit ;;
    --remove)     MODE=remove ;;
    --binary)     MODE=binary ;;
    --source)     MODE=source ;;
    --)           shift; break ;;
    -*)           internal "$cmd: bad option '$1'" ;;
    *)            break ;;
  esac
  shift
  done

  #
  #  Process arguments
  #

  [ "X$1" != X -a "X$2" = X ] || {
     echo "$cmd: wrong arg count (\$#=$#)"
     return1
  }
  PKG_NAME=$1
  [ "X$MKPKG_NAME" != Xunset ] || MKPKG_NAME=$PKG_NAME

  #  Check access to file because 'perl -pi -e' returns zero even
  # if the file does not exist!
  [ -f $EXTERNPKG_FILE ] || { echo "$EXTERNPKG_FILE: no such file"; return 1; }

  case $MODE in
  edit|source|binary)
    #  Check if the entry already exists in extern.pkg
    echo "$cmd: $PKG_NAME: check if entry exists ..."
    grep -qP "^reset\t$PKG_NAME\t" $EXTERNPKG_FILE && { echo "$EXTERNPKG_FILE: entry for $PKG_NAME already exist"; return 1; }

    #  Add the main entry in extern.pkg for the $PKG_NAME package.E.g. for ctio
    #  this adds:
    #
    #      reset   ctio            = iraf$ctio/
    #      task    ctio.pkg        = ctio$ctio.cl
    #
    #  (plus the trailing blank line).
    echo "$cmd: $PKG_NAME: adding main entry ..."
    perl -0777 -pi -e "s@^(# Manually)@reset\\t$PKG_NAME\\t\\t= $SCISOFT_SHARE/iraf/extern/$PKG_NAME/\\ntask\\t$PKG_NAME.pkg\\t= $PKG_NAME\\\$$PKG_NAME.cl\\n\\n\1@ms" $EXTERNPKG_FILE

    #  Add the help entry in extern.pkg for the $PKG_NAME package. E.g for ctio 
    #  this adds:
    #
    #      ,ctio$lib/helpdb.mip\
    echo "$cmd: $PKG_NAME: adding help entry ..."
    perl -0777 -pi -e "s@^(#[\\t ]+,example\\\$lib/helpdb\\.mip\\\\\\n)@\$1\\t\\t\\t  ,$PKG_NAME\\\$lib/helpdb.mip\\\\\\n@ms" $EXTERNPKG_FILE
    ;;

  remove)
    #  Check if the entry already exists in extern.pkg
    echo "$cmd: $PKG_NAME: check if entry exists ..."
    grep -qP "^reset\t$PKG_NAME\t" $EXTERNPKG_FILE || { echo "$EXTERNPKG_FILE: entry for $PKG_NAME already removed"; return 1; }

    #  Remove the main entry in extern.pkg for $PKG_NAME package.E.g. for ctio
    #  this will rvmove:
    #
    #      reset   ctio            = iraf$ctio/
    #      task    ctio.pkg        = ctio$ctio.cl
    #
    #  (plus the trailing blank line).
    echo "$cmd: $PKG_NAME: removing main entry ..."
    perl -0777 -pi -e "s@^reset\\t$PKG_NAME\\t\\t= $SCISOFT_SHARE/iraf/extern/$PKG_NAME/\\ntask\\t$PKG_NAME.pkg\\t= $PKG_NAME\\\$$PKG_NAME.cl\\n\\n@@ms" $EXTERNPKG_FILE
    #  Remove the help entry in extern.pkg for $PKG_NAME package. E.g for ctio 
    #  this removes:
    #
    #      ,ctio$lib/helpdb.mip\
    echo "$cmd: $PKG_NAME: removing help entry ..."
    perl -0777 -pi -e "s@^\\t\\t\\t  ,$PKG_NAME\\\$lib/helpdb.mip\\\\\\n@@ms" $EXTERNPKG_FILE
    return 0
    ;;
  esac

  case $MODE in
  source|binary)
    #  Call 'cl' and pass it the commands to make the package, either from 
    #  sources or binaries (well, actually binaries requires that the sources
    #  are also installed).

    echo "$cmd: $PKG_NAME: compiling $MODE package ..."
    {
      echo "$PKG_NAME"
      echo "cd $PKG_NAME"
      echo "softools"
      #  This sequence 'mkpkg' calls specified by Richard Hook in email 
      # to Alexis Huxley  (message id <4295D0D8.9080601@stsci.edu>).
      echo "mkpkg -p $MKPKG_NAME IRAFARCH_MARKER"
      [ $MODE = binary ] || {
        echo "mkpkg -p $PKG_NAME update"
      }
      echo "log"
      #  IRAF wants trailing slash on $iraf; without it 'cl' produces errors. 
      # 'cl' will fail to load system packages if not run in the same directory
      #  as the file login.cl.
    } | csh -c "cd; setenv iraf $SCISOFT_SHARE/iraf/iraf/; source \$iraf/unix/hlib/irafuser.csh; cl || exit 1; echo"
    ;;
  esac
  return 0
}

#
# Main
#
cmd=`basename $0`
export SCISOFT_BIN=`dirname $0`
export SCISOFT_ROOT=`dirname $SCISOFT_BIN`
export SCISOFT_SHARE=${SCISOFT_ROOT}/share

iraf_pkg $*
exit $?
